@@ -28,11 +28,14 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <time.h>
+#include <wait.h>
+#include <unistd.h>
 
 #include <qlayout.h>
 #include <qvbox.h>
 #include <qiconset.h> 
 #include <qsizepolicy.h> 
+#include <qevent.h>
 
 #include <kapplication.h>
 #include <kglobal.h>
@@ -49,7 +52,8 @@
 #include "statusbutton.h"
 #include "light.h"
 #include "graph.h"
-#include "modemlights.h"
+#include "speciallabels.h"
+#include "modem.h"
 #include "kmodemlightsdlg.h"
 #include "kmodemlights.h"
 #include "kmodemlights.moc"
@@ -59,67 +63,85 @@ extern "C" {
   {
     KGlobal::locale()->insertCatalogue("kmodemlights");
     return new KModemLightsApplet(configFile, KPanelApplet::Normal,
-			 KPanelApplet::About|KPanelApplet::Preferences,
-			 parent, "kmodemlightapplet");
+				  KPanelApplet::About|KPanelApplet::Preferences,
+				  parent, "kmodemlightapplet");
   }
 }
 
+extern char **environ;
+
+static int system_nowait(const char *command)
+{
+  if (command == 0)
+    return 1;
+  
+  /* go in "daemon mode" */
+  pid_t pid = fork();
+  if (pid < 0) return -1;	/* fork failed */
+  if (pid == 0) {    		/* we are the child */
+    pid_t cpid = fork();	/* fork the "daemon" */
+    if (cpid < 0) exit(-1);	/* fork failed */
+    if (cpid > 0) exit(0);	/* exit parent, to satify waiting process */
+    setsid();			/* found new processgroup */
+    const char * argv[4];
+    argv[0] = "sh";
+    argv[1] = "-c";
+    argv[2] = command;
+    argv[3] = 0;
+    execve("/bin/sh", (char *const *)argv, environ);
+    exit(127);
+  }
+  waitpid(pid, 0, 0);
+  return 0;
+}
+
 KModemLightsApplet::KModemLightsApplet(const QString& configFile, Type type, 
-				     int actions, QWidget *parent, 
-				     const char *name)
+				       int actions, QWidget *parent, 
+				       const char *name)
   : KPanelApplet(configFile, type, actions, parent, name),
     aboutData(0),
     timerID(0),
-    ffhAppletHeight(-1),
-    useISDN(false),
-    loadCounter(0),
+    timerEventCounter(0),
     lastTimeWasConnected(false),
     modemWasOn(false),
-    startTime(time(0)),
-    oldRxBytes(-1), RxBytesShowed(-1)
+    oldRxBytes(-1)
 {
   KGlobal::iconLoader()->addAppDir("kmodemlights");
 
-  socket = ::socket(AF_INET, SOCK_DGRAM, 0);
-
   // The applett itself
 
   // Button
   button = new StatusButton(this);
-  button->setStatus(StatusButton::Off);
-  button->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Minimum));
+  button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
   QObject::connect(button, SIGNAL(clicked()), this, SLOT(dial()));
 
   // graph and two lights
   lightRx = new Light(QColor(0x00FF00), QColor(0x004400), this);
+  lightRx->setBackgroundOrigin(AncestorOrigin);
   lightTx = new Light(QColor(0xFF0000), QColor(0x440000), this);
+  lightTx->setBackgroundOrigin(AncestorOrigin);
   graph = new Graph(this);
-  graph->setMinimumWidth(22);
-
-  QHBoxLayout *graphPart = new QHBoxLayout();
-  QVBoxLayout *vbox1 = new QVBoxLayout();
-  graphPart->addWidget(graph);
-  graphPart->addLayout(vbox1);
-  vbox1->addWidget(lightRx);
-  vbox1->addWidget(lightTx);
 
   // textual Displays
-  showRx = new QLabel("", this);
-  showRx->setAlignment(AlignRight|AlignVCenter);
-  showTime = new QLabel("*", this);
-  showTime->setAlignment(AlignRight|AlignVCenter);
-  QVBoxLayout *textualPart = new QVBoxLayout();
-  textualPart->addWidget(showRx);
-  textualPart->addWidget(showTime);
+  showRx = new TRLabel(this);
+  showRx->setBackgroundOrigin(AncestorOrigin);
+  showTime = new TRLabel(this);
+  showTime->setBackgroundOrigin(AncestorOrigin);
+  showTime->setTime(-1);
 
   // layouts
-  infoPart = new QBoxLayout(QBoxLayout::TopToBottom);
-  infoPart->addLayout(graphPart);
-  infoPart->addLayout(textualPart);
-  page = new QHBoxLayout(this);
-  page->setSpacing(1);
+  page = new QBoxLayout(this, QBoxLayout::LeftToRight, 0, 1);
+  page->setResizeMode(QLayout::FreeResize);
   page->addWidget(button);
-  page->addLayout(infoPart);
+  infoPart = new QBoxLayout(page, QBoxLayout::TopToBottom, 1);
+  QHBoxLayout *graphPart = new QHBoxLayout(infoPart, 0);
+  graphPart->addWidget(graph);
+  QVBoxLayout *vbox1 = new QVBoxLayout(graphPart, 0);
+  vbox1->addWidget(lightRx);
+  vbox1->addWidget(lightTx);
+  QVBoxLayout *textualPart = new QVBoxLayout(infoPart, 0);
+  textualPart->addWidget(showRx);
+  textualPart->addWidget(showTime);
 
   // popupMenu
   popup = new KPopupMenu();
@@ -138,67 +160,142 @@ KModemLightsApplet::KModemLightsApplet(const QString& configFile, Type type,
 KModemLightsApplet::~KModemLightsApplet()
 {
   killTimer(timerID);
-#if 1
-  if(socket != -1) ::close(socket);
-#endif
   if(aboutData) delete aboutData;
 }
 
 int KModemLightsApplet::widthForHeight(int height) const
 {
-  int fontheight, widgetheight;
+  // two-row-layout or single-row-layout
+  int widgetheight = height >= 42 ? (height-1)/4 : height/2;
+  
+  // button
+  int b = button->sizeHint().width();
 
-  // calc Fontheight
-  if (height>=42) {	// two-row-layout
+  // graph + lights
+  int w1 = 22 + (widgetheight > 12 ? 12 : 9); 
+ 
+  // take the configured KDE fixed font
+  QFont font(KGlobalSettings::fixedFont());
+  font.setPixelSize(widgetheight);
+  // guess the maximum size by calulate the size of a string set to 
+  // 4 digits and 2 characters (b,m,' or ")
+  int w2 = (QFontMetrics(font).width("88m88m"));
+
+  if (height >= 42) // two-row-layout 
+    return b + page->spacing() + (w1>w2 ? w1 : w2);
+  else  // single-row-layout
+    return b + page->spacing() + w1 + infoPart->spacing() + w2;
+}
+
+static inline int fontHeightForWidth(int low, int up, int w, 
+				     const QString s, QFont f)
+{
+  f.setPixelSize(up);
+  int upWidth = QFontMetrics(f).width(s);
+  if (upWidth < w) return up;
+  f.setPixelSize(low);
+  int lowWidth = QFontMetrics(f).width(s);
+  if (lowWidth > w) return low;
+  do {
+    if (up == low || up == low+1) break;
+    int mid = (up+low)/2;
+    f.setPixelSize(mid);
+    int midWidth = QFontMetrics(f).width(s);
+    if (midWidth < w) {
+      lowWidth = midWidth;
+      low = mid;
+    } else {
+      upWidth = midWidth;
+      up = mid;
+    }
+  } while(1);
+  return  low;
+}
+
+int KModemLightsApplet::heightForWidth(int width) const
+{
+  // button
+  int b = button->sizeHint().height();
+
+  // lights + graph
+  int h1 = 2*9;
+  
+  // take the configured KDE fixed font
+  QFont font(KGlobalSettings::fixedFont());
+  int h2 = 2 * fontHeightForWidth(4, 48, width, "88m88m", font);
+
+  return b + page->spacing() + h1 + infoPart->spacing() + h2;
+}
+
+void KModemLightsApplet::resizeEvent(QResizeEvent *event)
+{
+  switch(position()) {
+  case pTop:
+  case pBottom:
+    horizontalLayout(height());
+    break;
+  case pLeft:
+  case pRight:
+    verticalLayout(width());
+    break;
+  default:
+    return;
+  }
+  QWidget::resizeEvent(event);
+}
+
+void KModemLightsApplet::horizontalLayout(int height)
+{
+  int widgetheight;
+  if(page->direction() != QBoxLayout::LeftToRight)
+    page->setDirection(QBoxLayout::LeftToRight);
+  if (height >= 42) {	// two-row-layout
     widgetheight = height/4;
-    fontheight = widgetheight;
     if(infoPart->direction() != QBoxLayout::TopToBottom)
       infoPart->setDirection(QBoxLayout::TopToBottom);
   } else {		// single-row-layout
     widgetheight = height/2;
-    fontheight = widgetheight;
     if(infoPart->direction() != QBoxLayout::LeftToRight)
-      infoPart->setDirection(QBoxLayout::LeftToRight);    
+      infoPart->setDirection(QBoxLayout::LeftToRight); 
   }
-
-  // take the configured KDE fixed font
-  QFont font(KGlobalSettings::fixedFont());
-  font.setPixelSize(fontheight);
-  // and set it on the text-widgets
-  showRx->setFont(font);
-  showRx->setMaximumHeight(widgetheight);
-  showTime->setFont(font);
-  showTime->setMaximumHeight(widgetheight);
-  
-  if(widgetheight>12) {
+      
+  if(widgetheight > 12) {
     lightRx->setBig(true);
     lightTx->setBig(true);
   } else {
     lightRx->setBig(false);
     lightTx->setBig(false);
   }
-
-  // guess the maximum size by setting a label to 
-  // 4 digits 2 characters(b,m,' or ")
-  showTime->setText("00'00\"");
-
-  return this->sizeHint().width();;
+      
+  // take the configured KDE fixed font
+  QFont font(KGlobalSettings::fixedFont());
+  font.setPixelSize(widgetheight);
+  //bmg  int textwidth = QFontMetrics(font).width("88m88m");
+  // and set it on the text-widgets
+  showRx->setFont(font);
+  showRx->setFixedHeight(widgetheight);
+  showTime->setFont(font);
+  showTime->setFixedHeight(widgetheight);
 }
 
-int KModemLightsApplet::heightForWidth(int) const
+void KModemLightsApplet::verticalLayout(int width)
 {
-  // fixme: this does not work yet
-  // there is no concept for vertical layout at all
-
+  if(page->direction() != QBoxLayout::TopToBottom)
+    page->setDirection(QBoxLayout::TopToBottom);
+  if(infoPart->direction() != QBoxLayout::TopToBottom)
+    infoPart->setDirection(QBoxLayout::TopToBottom);
+  lightRx->setBig(false);
+  lightTx->setBig(false);
+      
+  // take the configured KDE fixed font
   QFont font(KGlobalSettings::fixedFont());
-  font.setPixelSize(10);
-
+  int fontheight = fontHeightForWidth(4, 48, width, "88m88m", font);
+  font.setPixelSize(fontheight);
+  // and set it on the text-widgets
   showRx->setFont(font);
-  showRx->setMaximumHeight(10);
+  showRx->setFixedHeight(fontheight);
   showTime->setFont(font);
-  showTime->setMaximumHeight(10);
-
-  return 42;
+  showTime->setFixedHeight(fontheight);
 }
 
 void KModemLightsApplet::mousePressEvent(QMouseEvent *e)
@@ -209,67 +306,50 @@ void KModemLightsApplet::mousePressEvent(QMouseEvent *e)
   }
 }
 
-static inline QString formatBytes(int n)
-{
-  if(n<10000) 
-    return QString().sprintf("%4db", n);
-  else if(n<10000000) 
-    return QString().sprintf("%4dK", n/1024);
-  else
-    return QString().sprintf("%4dM", n/(1024*1024));
-}
-
 void KModemLightsApplet::timerEvent(QTimerEvent *)
 {
-  int rx, tx;
-  bool lightRx = false;
-  bool lightTx = false;
+  unsigned int rx, tx;
 
-  loadCounter++;
+  timerEventCounter++;
+
+  if (modem.isConnected()) {
+    bool statsRet = modem.getStats(rx, tx);
 
-  if (isConnected()) {
     // Timer
     if (!lastTimeWasConnected) {
-      getConnectTime(true); // reset start time
-      lastTimeWasConnected = TRUE;
+      modem.getConnectTime(true); // reset start time
+      lastTimeWasConnected = true;
     }
     updateTimer();
 
     // button and lights
-    if (!getStats(rx, tx) || (rx == 0 && tx == 0)) {
-      oldRx = oldTx = 0;
+    if (!statsRet || (rx == 0 && tx == 0)) {
       button->setStatus(StatusButton::Wait);
+      oldRx = oldTx = 0;
+      updateLights(false, false);
     } else {
       button->setStatus(StatusButton::On);
-      if (rx > oldRx) lightRx = true;
-      if (tx > oldTx) lightTx = true;
+      updateLights((rx > oldRx), (tx > oldTx));
+      oldRx = rx;
+      oldTx = tx;
     }
-    updateLights(lightRx, lightTx);
-    oldRx = rx;
-    oldTx = tx;
 
     // (all 1 sec)
-    if (loadCounter % updateFrequency == 0) {
-      updateRx(rx);
+    if (timerEventCounter % updateFrequency == 0) {
+      if (statsRet)
+	updateRx(rx);
 
       // (all 2 sec)
-      if (loadCounter % (updateFrequency * 2) == 0) {
-	
-	// (all 20 sec)
-	if (loadCounter % (updateFrequency * 20) == 0) {
-	  //	update_tooltip(TRUE, rx, tx);
-	}
-	
+      if (timerEventCounter % (updateFrequency * 2) == 0) {	
 	if (!modemWasOn) {
-	  loadRx = rx; // reset loadRx/loadTx
-	  loadTx = tx;
-	  //	update_tooltip(TRUE, rx, tx);
-	  modemWasOn = TRUE;
+	  graphRx = rx; // reset graphRx/graphTx
+	  graphTx = tx;
+	  modemWasOn = true;
 	}
 	
-	updateLoad(rx-loadRx, tx-loadTx);
-	loadRx = rx;
-	loadTx = tx;
+	updateGraph(rx-graphRx, tx-graphTx);
+	graphRx = rx;
+	graphTx = tx;
       }
     }
   } else {
@@ -278,49 +358,21 @@ void KModemLightsApplet::timerEvent(QTimerEvent *)
     updateLights(false, false);
     
     // (all 2 sec)
-    if (loadCounter % (updateFrequency * 2) == 0) {
-      updateLoad(0, 0);
+    if (timerEventCounter % (updateFrequency * 2) == 0) {
+      updateGraph(0, 0);
     }
-    if (modemWasOn) {
-      //      update_tooltip(FALSE,0,0);
-      modemWasOn = FALSE;
-    }
-    if (lastTimeWasConnected) lastTimeWasConnected = FALSE;
+    if (modemWasOn) modemWasOn = false;
+    if (lastTimeWasConnected) lastTimeWasConnected = false;
   }
 }
 
-
-bool KModemLightsApplet::isConnected()
-{
-  if (useISDN)
-    return is_ISDN_on();
-  else
-    return is_Modem_on(lockFile.ascii());
-}
-
-bool KModemLightsApplet::getStats(int &in, int &out)
-{
-  if (useISDN)
-    return get_ISDN_stats(&in, &out);
-  else
-    return get_modem_stats(&in, &out, socket, deviceName.ascii());
-}
-
-int KModemLightsApplet::getConnectTime(bool recalcStart)
-{
-  if (useISDN)
-    return get_ISDN_connect_time(recalcStart);
-  else
-    return get_modem_connect_time(recalcStart, lockFile.ascii());
-}
-
 void KModemLightsApplet::updateLights(bool rx, bool tx)
 {
   lightRx->setStatus(rx);
   lightTx->setStatus(tx);
 }
 
-void KModemLightsApplet::updateLoad(unsigned int rx, unsigned int tx)
+void KModemLightsApplet::updateGraph(unsigned int rx, unsigned int tx)
 {
   graph->addValues(rx, tx);
 }
@@ -347,45 +399,27 @@ void KModemLightsApplet::updateRx(int rxBytes)
   int rx = rxBytes - oldRxBytes;
   oldRxBytes = rxBytes;
 
-  if (rx == RxBytesShowed) return;
-
-  showRx->setText(formatBytes(rx));
-  RxBytesShowed = rx;
-
+  showRx->setBytes(rx);
 }
 
 void  KModemLightsApplet::updateTimer()
 {
-  int newTimer = getConnectTime(false);
-  if (newTimer != oldTimer) {
-    if (newTimer>3600) { /* HH:MM */
-      newTimer /= 60; 
-      int a = newTimer / 60;
-      int b = newTimer % 60;
-      showTime->setText(QString().sprintf("%02d:%02d", a, b));
-    } else {
-      int a = newTimer / 60;
-      int b = newTimer % 60;
-      
-      //showTime->setText(QString().sprintf("%02d%s%02d%s", a, "ʹ", b, "ʺ"));
-      showTime->setText(QString().sprintf("%02d%s%02d%s", a, "'", b, "\""));
-    }
-    oldTimer = newTimer;
-  }
+  int newTimer = modem.getConnectTime(false);
+  showTime->setTime(newTimer);
 }
 
 void KModemLightsApplet::dial()
 {
-  if (isConnected()) {
+  if (modem.isConnected()) {
     int ans = KMessageBox::questionYesNo(this, 
 					 i18n("You are currently connected.\n"
 					      "Do you want to disconnect?"));
     if (ans == KMessageBox::Yes)
-      system(disconnectCommand.ascii());
+      system_nowait(disconnectCommand.ascii());
   } else {
     int ans = KMessageBox::questionYesNo(this, i18n("Do you want to connect?"));
     if (ans == KMessageBox::Yes)
-    system(connectCommand.ascii());
+      system_nowait(connectCommand.ascii());
   }
 }
 
@@ -448,12 +482,16 @@ void KModemLightsApplet::readConfig()
   config()->setGroup("General");
 
   // read config
-  lockFile = config()->readPathEntry("Lockfile", "/var/lock/LCK..ttyS1");
-  deviceName = config()->readEntry("Devicename", "ppp0");
   connectCommand = config()->readEntry("Connect", "pon");
   disconnectCommand = config()->readEntry("Disconnect", "poff");
   updateFrequency = config()->readNumEntry("Update frequency", 5);
 
+  // read modem config
+  QString lockFile = config()->readPathEntry("Lockfile", "/var/lock/LCK..ttyS1");
+  modem.setLockFile(lockFile.ascii());
+  QString deviceName = config()->readEntry("Devicename", "ppp0");
+  modem.setDeviceName(deviceName.ascii());
+
   // default colors
   static const QColor CGreen(0x00FF00);
   static const QColor CRed(0xFF0000);
