@@ -28,15 +28,16 @@
 #include <qlabel.h>
 
 #include "statusbutton.h"
+#include "modem.h"
 #include "kpanelapplet.h"
 
 
-class QVBoxLayout;
 class MapWidget;
 class KAboutData;
 class KPopupMenu;
 class Light;
 class Graph;
+class TRLabel;
 
 class KModemLightsApplet : public KPanelApplet
 {
@@ -51,7 +52,6 @@ class KModemLightsApplet : public KPanelApplet
 
   int widthForHeight(int height) const;
   int heightForWidth(int width) const;
-  bool getStats(int &in, int &out) const;
 
  public slots:
 
@@ -60,9 +60,6 @@ class KModemLightsApplet : public KPanelApplet
  protected:
 
   void mousePressEvent(QMouseEvent *);
-  bool isConnected();
-  bool getStats(int &in, int &out);
-  int getConnectTime(bool recalcStart);
 
  protected slots:
  
@@ -70,26 +67,29 @@ class KModemLightsApplet : public KPanelApplet
   void about();
   void preferences();
 
+  virtual void resizeEvent(QResizeEvent *);  
+
  private:
 
   void readConfig();
 
   void updateTimer();
   void updateLights(bool rx, bool tx);
-  void updateLoad(unsigned int rx, unsigned int tx);
+  void updateGraph(unsigned int rx, unsigned int tx);
   void updateRx(int rx);
-  
-  // Elements of applet
+
+  void horizontalLayout(int height);
+  void verticalLayout(int width);
+
+  // elements of applet
   StatusButton *button;
   Graph *graph;
   Light *lightRx, *lightTx;
-  QLabel *showRx;
-  QLabel *showTime;
-  // layout
-  QHBoxLayout *page;
+  TRLabel *showRx;
+  TRLabel *showTime;
+  // layouts
+  QBoxLayout *page;
   QBoxLayout *infoPart;
-  //  QHBoxLayout *graphPart;
-  //  QVBoxLayout *textualPart;
 
   KAboutData *aboutData;
   KPopupMenu *popup;
@@ -97,37 +97,24 @@ class KModemLightsApplet : public KPanelApplet
   int timerID;
 
   // Configurabel Entities
-  QString lockFile;
-  QString deviceName;
   QString connectCommand;
   QString disconnectCommand;
   int updateFrequency;
+  Modem modem;
 
-  // fonts for height
-  mutable int ffhAppletHeight, ffhFontSize;
-
-  bool useISDN;
-
-  int socket;
-  bool connectedLast;
-  bool upLast;
-  int oldRx, oldTx;
-  int loadRx, loadTx;
-  unsigned int loadCounter;
+  // memory for timerEvent
+  unsigned int timerEventCounter;
   bool lastTimeWasConnected;
   bool modemWasOn;
-  time_t startTime;
-  int oldTimer;
-  int oldRxBytes, RxBytesShowed;
-  
+  // memory for lights
+  unsigned int oldRx, oldTx;
+  // memory for graph
+  int graphRx, graphTx;
+  // memory for updateRx
+  int oldRxBytes;
+
   friend class KModemLightsDlg;
 
 };
 
 #endif
-
-/*
-is_connected:  is_ISDN_on() || is_Modem_on()
-get_stats:     get_ISDN_stats || get_modem_stats
-get_connect_time  get_ISDN_connect_time || get_modem_connect_time
-*/
