@@ -46,7 +46,6 @@ KModemLightsDlg::KModemLightsDlg(QWidget *parent, const char *name,
 		Ok|Cancel|Help),
     applet(ap)
 {
-
   KTabWidget *page = new KTabWidget(this);
   page->setMargin(marginHint());
   setMainWidget(page);
@@ -64,54 +63,66 @@ QWidget *KModemLightsDlg::generateDevice()
   QGrid *grid = new QGrid(2, tab);
   grid->setSpacing(spacingHint());
   vbox->add(grid);
-  vbox->addStretch();
 
   // Frequency
   QLabel *label = new QLabel(i18n("Update Frequency:"), grid, "caption" );
   frequencyWidget = new QSpinBox(1, 5, 1, grid);
   frequencyWidget->setValue(applet->updateFrequency);
   frequencyWidget->setSuffix(" Hz");
-  QString text = i18n("This is the frequency the applet is updated.\n"
-		      "A value of 3 means it is updated 3 times a second."
-		      "The default value is 5. On extremly slow systems"
+  QString text = i18n("<b>This is the frequency the applet is updated.</b>"
+		      "<p>A value of 3 means it is updated 3 times a second."
+		      "The default value is 5. On extremly slow systems "
 		      "you might like to reduce the frequency to lower "
-		      "the system-load, mostly the default is O.K.");
+		      "the system-load, mostly the default is O.K.</p>");
   QWhatsThis::add(label, text);
   QWhatsThis::add(frequencyWidget, text);
 
   // Device Name
   label = new QLabel(i18n("Device:"), grid);
-  text = i18n("This is the name of the device, p.e. 'ppp0' "
-	      "for (the first) PPP device.");
+  text = i18n("<b>This is the name of the device.</b>"
+	      "<p>P.e. 'ppp0' for (the first) PPP device.</p>");
   QWhatsThis::add(label, text);  
-  deviceWidget = new QLineEdit(applet->deviceName, grid);
+  deviceWidget = new QLineEdit(applet->modem.deviceName(), grid);
   QWhatsThis::add(deviceWidget, text);
   
   // Lockfile Name
   label = new QLabel(i18n("Lockfile:"), grid);
-  text = i18n("This is the path to the file, that is used to lock "
-	      "the device, On a standard linux-system this is "
-	      "'/var/lock/LCK..ttyS1' for a modem on '/dev/ttyS1'.");
+  text = i18n("<b>This is the path to the file, that is used to lock"
+	      "the device.</b>"
+	      "<p>On a standard linux-system this is "
+	      "<nobr>'/var/lock/LCK..ttyS1'</nobr> for a modem "
+	      "on <nobr>'/dev/ttyS1'</nobr>.</p>");
   QWhatsThis::add(label, text);
-  lockFileWidget = new QLineEdit(applet->lockFile, grid);
+  lockFileWidget = new QLineEdit(applet->modem.lockFile(), grid);
   QWhatsThis::add(lockFileWidget, text);
 
   // Connect command
   label = new QLabel(i18n("Connect:"), grid);
-  text = i18n("This is the command that is executed to go online.\n"
-	      "It is different on different distributions and systems.");
+  text = i18n("<b>This is the command that is executed to go online.</b>"
+	      "<p>It is different on different distributions and systems.</p>" 
+	      "<p>For a connection configured with kppp it is "
+	      "<nobr>'kppp -c &lt;provider&gt;'</nobr>.</p>");
   QWhatsThis::add(label, text);
   connectWidget = new QLineEdit(applet->connectCommand, grid);
   QWhatsThis::add(connectWidget, text);
 
   // Disonnect command
   label = new QLabel(i18n("Disconnect:"), grid);
-  text = i18n("This is the command that is executed to go offline.\n"
-	      "It is different on different distributions and systems.");
+  text = i18n("<b>This is the command that is executed to go offline.</b>"
+	      "<p>It is different on different distributions and systems.</p>"
+	      "<p>For a connection configured with kppp it is "
+	      "<nobr>'kppp -k -q'<nobr>.</p>");
   QWhatsThis::add(label, text);
   disconnectWidget = new QLineEdit(applet->disconnectCommand, grid);
   QWhatsThis::add(disconnectWidget, text);
 
+  // Hint to use What's this help
+  vbox->addStretch();
+  label = new QLabel(i18n("<i>There is help available for all fields on this "
+			  "tab, that can be shown with Shift-F1.</i>"), tab);
+  label->setTextFormat(Qt::RichText); // does not break the text otherwise
+  vbox->addWidget(label);
+
   return tab;
 }
 
@@ -129,9 +140,8 @@ QWidget *KModemLightsDlg::generateColor()
   QVBoxLayout *vbox = new QVBoxLayout(tab);
   vbox->setSpacing(spacingHint());
 
-  QHBoxLayout *hbox = new QHBoxLayout(vbox);
   QGroupBox *group = new QGroupBox(2, Qt::Vertical, i18n("Receive Data"), tab);
-  hbox->add(group);
+  vbox->add(group);
   rxOn = new KColorButton(group);
   QLabel *label = newCaption(i18n("On"), group);
   rxOff = new KColorButton(group);
@@ -139,9 +149,8 @@ QWidget *KModemLightsDlg::generateColor()
   rxGraph = new KColorButton(group);
   label = newCaption(i18n("Graph"), group);
 
-  hbox = new QHBoxLayout(vbox);
   group = new QGroupBox(2, Qt::Vertical, i18n("Send Data"), tab);
-  hbox->add(group);
+  vbox->add(group);
   txOn = new KColorButton(group);
   label = newCaption(i18n("On"), group);
   txOff = new KColorButton(group);
@@ -149,9 +158,8 @@ QWidget *KModemLightsDlg::generateColor()
   txGraph = new KColorButton(group);
   label = newCaption(i18n("Graph"), group);
 
-  hbox = new QHBoxLayout(vbox);
   group = new QGroupBox(2, Qt::Vertical, i18n("Connection Status"), tab);
-  hbox->add(group);
+  vbox->add(group);
   statusAwait = new KColorButton(group);
   label = newCaption(i18n("Await Connection"), group);
   statusOn = new KColorButton(group);
@@ -171,8 +179,6 @@ QWidget *KModemLightsDlg::generateColor()
   statusAwait->setColor(applet->button->getColor(StatusButton::Wait));
   statusOn->setColor(applet->button->getColor(StatusButton::On));
 
-  //  connect(button, SIGNAL(clicked()), SLOT(fontcb()));
-
   return tab;
 }
 
