@@ -33,7 +33,8 @@
 QPixmap Light::lightPixmapS, Light::lightMaskS;  
 QPixmap Light::lightPixmapL, Light::lightMaskL;
 
-Light::Light(const QColor &cOn, const QColor &cOff, QWidget *parent, const char *name)
+Light::Light(const QColor &cOn, const QColor &cOff, 
+	     QWidget *parent, const char *name)
   : QLabel(parent, name),
     lightStatus(false), bigLights(false),
     colorOn(cOn), colorOff(cOff)
@@ -49,7 +50,8 @@ Light::Light(const QColor &cOn, const QColor &cOff, QWidget *parent, const char
   generateIcon(on, colorOn);
   generateIcon(off, colorOff);
 
-  setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
+  setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+  setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
 
   setPixmap(off);
 }
