@@ -33,19 +33,25 @@ class Light : public QLabel
   Light(const QColor &cOn, const QColor &cOff, QWidget *parent, const char *name = 0);
 
   void setStatus(bool s);
-  bool status() const { return _status; };
+  bool status() const { return lightStatus; };
   void setColors(const QColor &cOn, const QColor &cOff);
   QColor getColorOn() const { return colorOn; };
   QColor getColorOff() const { return colorOff; };
+  void setBig(bool b);
+  bool big() const { return bigLights; };
 
  private:
 
   void generateIcon(QPixmap &px, QColor &col);
 
-  bool _status;
+  bool lightStatus;
+  bool bigLights;
   QPixmap on, off;
-  QPixmap lightPixmap, lightMask;
   QColor colorOn, colorOff;
+
+  // one version of these is enough;
+  static QPixmap lightPixmapS, lightMaskS;
+  static QPixmap lightPixmapL, lightMaskL;
 };
 
 #endif
