@@ -117,7 +117,7 @@ void StatusButton::setColor(const Status s, const QColor &c)
   }
 }
 
-QColor StatusButton::getColor(const Status s)
+QColor StatusButton::getColor(const Status s) const
 {
   switch(s) {
   case On:
@@ -135,7 +135,7 @@ QColor StatusButton::getColor(const Status s)
   }
 }
 
-void StatusButton::generateIcon(QPixmap &px, QPixmap &sign, QColor &col)
+void StatusButton::generateIcon(QPixmap &px, const QPixmap &sign, const QColor &col)
 {
   px = signMask;
   QPainter p(&px);
